@@ -144,7 +144,7 @@ unused_var_in_catch: {
         function foo() {
             try {
                 foo();
-            } catch(ex) {
+            } catch (ex) {
                 var x = 10;
             }
         }
@@ -153,7 +153,7 @@ unused_var_in_catch: {
         function foo() {
             try {
                 foo();
-            } catch(ex) {}
+            } catch (ex) {}
         }
     }
 }
@@ -166,7 +166,7 @@ used_var_in_catch: {
         function foo() {
             try {
                 foo();
-            } catch(ex) {
+            } catch (ex) {
                 var x = 10;
             }
             return x;
@@ -176,7 +176,7 @@ used_var_in_catch: {
         function foo() {
             try {
                 foo();
-            } catch(ex) {
+            } catch (ex) {
                 var x = 10;
             }
             return x;
@@ -797,6 +797,7 @@ assign_chain: {
 issue_1583: {
     options = {
         keep_fargs: true,
+        passes: 2,
         reduce_funcs: true,
         reduce_vars: true,
         unused: true,
@@ -1004,7 +1005,7 @@ issue_1715_4: {
 delete_assign_1: {
     options = {
         booleans: true,
-        side_effects: true,
+        evaluate: true,
         toplevel: true,
         unused: true,
     }
@@ -1023,7 +1024,7 @@ delete_assign_1: {
         console.log((1 / 0, !0));
         console.log((1 / 0, !0));
         console.log((NaN, !0));
-        console.log((0 / 0, !0));
+        console.log((NaN, !0));
     }
     expect_stdout: true
 }
@@ -1031,8 +1032,8 @@ delete_assign_1: {
 delete_assign_2: {
     options = {
         booleans: true,
+        evaluate: true,
         keep_infinity: true,
-        side_effects: true,
         toplevel: true,
         unused: true,
     }
@@ -1051,7 +1052,7 @@ delete_assign_2: {
         console.log((Infinity, !0));
         console.log((1 / 0, !0));
         console.log((NaN, !0));
-        console.log((0 / 0, !0));
+        console.log((NaN, !0));
     }
     expect_stdout: true
 }
@@ -1144,6 +1145,7 @@ var_catch_toplevel: {
     options = {
         conditionals: true,
         negate_iife: true,
+        passes: 2,
         reduce_funcs: true,
         reduce_vars: true,
         side_effects: true,
@@ -1156,7 +1158,7 @@ var_catch_toplevel: {
             try {
                 a++;
                 x();
-            } catch(a) {
+            } catch (a) {
                 if (a) var a;
                 var a = 10;
             }
@@ -1167,7 +1169,7 @@ var_catch_toplevel: {
         !function() {
             try {
                 x();
-            } catch(a) {
+            } catch (a) {
                 var a;
             }
         }();
@@ -2005,3 +2007,58 @@ issue_3233: {
     }
     expect_stdout: "PASS"
 }
+
+issue_3375: {
+    options = {
+        reduce_vars: true,
+        toplevel: true,
+        unused: true,
+    }
+    input: {
+        var b = 1;
+        var a = c = [], c = --b + ("function" == typeof f && f());
+        var a = c && c[a];
+        console.log(a, b);
+    }
+    expect: {
+        var b = 1;
+        var a = [], c = --b + ("function" == typeof f && f());
+        a = c && c[a];
+        console.log(a, b);
+    }
+    expect_stdout: "0 0"
+}
+
+issue_3427_1: {
+    options = {
+        sequences: true,
+        side_effects: true,
+        unused: true,
+    }
+    input: {
+        (function() {
+            var a;
+            a = a || {};
+        })();
+    }
+    expect: {}
+}
+
+issue_3427_2: {
+    options = {
+        unused: true,
+    }
+    input: {
+        (function() {
+            var s = "PASS";
+            console.log(s = s || "FAIL");
+        })();
+    }
+    expect: {
+        (function() {
+            var s = "PASS";
+            console.log(s = s || "FAIL");
+        })();
+    }
+    expect_stdout: "PASS"
+}
