@@ -2675,3 +2675,476 @@ cross_references_3: {
         "9 27",
     ]
 }
+
+loop_inline: {
+    options = {
+        inline: true,
+        reduce_vars: true,
+        unused: true,
+    }
+    input: {
+        console.log(function(o) {
+            function g(p) {
+                return o[p];
+            }
+            function h(q) {
+                while (g(q));
+            }
+            return h;
+        }([ 1, "foo", 0 ])(2));
+    }
+    expect: {
+        console.log(function(o) {
+            return function(q) {
+                while (p = q, o[p]);
+                var p;
+            };
+        }([ 1, "foo", 0 ])(2));
+    }
+    expect_stdout: "undefined"
+}
+
+functions: {
+    options = {
+        functions: true,
+        reduce_vars: true,
+        unused: true,
+    }
+    input: {
+        !function() {
+            var a = function a() {
+                return a && "a";
+            };
+            var b = function x() {
+                return !!x;
+            };
+            var c = function(c) {
+                return c;
+            };
+            if (c(b(a()))) {
+                var d = function() {};
+                var e = function y() {
+                    return typeof y;
+                };
+                var f = function(f) {
+                    return f;
+                };
+                console.log(a(d()), b(e()), c(f(42)), typeof d, e(), typeof f);
+            }
+        }();
+    }
+    expect: {
+        !function() {
+            function a() {
+                return a && "a";
+            }
+            function b() {
+                return !!b;
+            }
+            var c = function(c) {
+                return c;
+            };
+            if (c(b(a()))) {
+                function d() {}
+                function e() {
+                    return typeof e;
+                }
+                var f = function(f) {
+                    return f;
+                };
+                console.log(a(d()), b(e()), c(f(42)), typeof d, e(), typeof f);
+            }
+        }();
+    }
+    expect_stdout: "a true 42 function function function"
+}
+
+functions_use_strict: {
+    options = {
+        functions: true,
+        reduce_vars: true,
+        unused: true,
+    }
+    input: {
+        "use strict";
+        !function() {
+            var a = function a() {
+                return a && "a";
+            };
+            var b = function x() {
+                return !!x;
+            };
+            var c = function(c) {
+                return c;
+            };
+            if (c(b(a()))) {
+                var d = function() {};
+                var e = function y() {
+                    return typeof y;
+                };
+                var f = function(f) {
+                    return f;
+                };
+                console.log(a(d()), b(e()), c(f(42)), typeof d, e(), typeof f);
+            }
+        }();
+    }
+    expect: {
+        "use strict";
+        !function() {
+            function a() {
+                return a && "a";
+            }
+            function b() {
+                return !!b;
+            }
+            var c = function(c) {
+                return c;
+            };
+            if (c(b(a()))) {
+                var d = function() {};
+                var e = function y() {
+                    return typeof y;
+                };
+                var f = function(f) {
+                    return f;
+                };
+                console.log(a(d()), b(e()), c(f(42)), typeof d, e(), typeof f);
+            }
+        }();
+    }
+    expect_stdout: "a true 42 function function function"
+}
+
+issue_2437: {
+    options = {
+        collapse_vars: true,
+        conditionals: true,
+        functions: true,
+        inline: true,
+        join_vars: true,
+        passes: 2,
+        reduce_funcs: true,
+        reduce_vars: true,
+        sequences: true,
+        side_effects: true,
+        toplevel: true,
+        unused: true,
+    }
+    input: {
+        function foo() {
+            return bar();
+        }
+        function bar() {
+            if (xhrDesc) {
+                var req = new XMLHttpRequest();
+                var result = !!req.onreadystatechange;
+                Object.defineProperty(XMLHttpRequest.prototype, 'onreadystatechange', xhrDesc || {});
+                return result;
+            } else {
+                var req = new XMLHttpRequest();
+                var detectFunc = function(){};
+                req.onreadystatechange = detectFunc;
+                var result = req[SYMBOL_FAKE_ONREADYSTATECHANGE_1] === detectFunc;
+                req.onreadystatechange = null;
+                return result;
+            }
+        }
+        console.log(foo());
+    }
+    expect: {
+        console.log(function() {
+            if (xhrDesc) {
+                var result = !!(req = new XMLHttpRequest()).onreadystatechange;
+                return Object.defineProperty(XMLHttpRequest.prototype, "onreadystatechange", xhrDesc || {}),
+                    result;
+            }
+            function detectFunc() {}
+            var req = new XMLHttpRequest();
+            return req.onreadystatechange = detectFunc,
+            result = req[SYMBOL_FAKE_ONREADYSTATECHANGE_1] === detectFunc,
+            req.onreadystatechange = null, result;
+        }());
+    }
+}
+
+issue_2485: {
+    options = {
+        functions: true,
+        reduce_funcs: true,
+        reduce_vars: true,
+        unused: true,
+    }
+    input: {
+        var foo = function(bar) {
+            var n = function(a, b) {
+                return a + b;
+            };
+            var sumAll = function(arg) {
+                return arg.reduce(n, 0);
+            };
+            var runSumAll = function(arg) {
+                return sumAll(arg);
+            };
+            bar.baz = function(arg) {
+                var n = runSumAll(arg);
+                return (n.get = 1), n;
+            };
+            return bar;
+        };
+        var bar = foo({});
+        console.log(bar.baz([1, 2, 3]));
+    }
+    expect: {
+        var foo = function(bar) {
+            function n(a, b) {
+                return a + b;
+            }
+            function runSumAll(arg) {
+                return function(arg) {
+                    return arg.reduce(n, 0);
+                }(arg);
+            }
+            bar.baz = function(arg) {
+                var n = runSumAll(arg);
+                return (n.get = 1), n;
+            };
+            return bar;
+        };
+        var bar = foo({});
+        console.log(bar.baz([1, 2, 3]));
+    }
+    expect_stdout: "6"
+}
+
+issue_3364: {
+    options = {
+        functions: true,
+        reduce_vars: true,
+        toplevel: true,
+        unused: true,
+    }
+    mangle = {}
+    input: {
+        var s = 2, a = 100, b = 10, c = 0;
+        function f(p, e, r) {
+            try {
+                for (var i = 1; i-- > 0;)
+                    var a = function(x) {
+                        function g(y) {
+                            y && y[a++];
+                        }
+                        var x = g(--s >= 0 && f(c++));
+                        for (var j = 1; --j > 0;);
+                    }();
+            } catch (e) {
+                try {
+                    return;
+                } catch (z) {
+                    for (var k = 1; --k > 0;) {
+                        for (var l = 1; l > 0; --l) {
+                            var n = function() {};
+                            for (var k in n)
+                                var o = (n, k);
+                        }
+                    }
+                }
+            }
+        }
+        var r = f();
+        console.log(c);
+    }
+    expect: {
+        var s = 2, c = 0;
+        (function n(r, o, a) {
+            try {
+                for (var f = 1; f-- >0;)
+                    var t = function(r) {
+                        (function(r) {
+                            r && r[t++];
+                        })(--s >= 0 && n(c++));
+                        for (var o = 1; --o > 0;);
+                    }();
+            } catch (o) {
+                try {
+                    return;
+                } catch (r) {
+                    for (var v = 1; --v > 0;)
+                        for (var i = 1; i > 0;--i) {
+                            function u() {}
+                            for (var v in u);
+                        }
+                }
+            }
+        })();
+        console.log(c);
+    }
+    expect_stdout: "2"
+}
+
+issue_3366: {
+    options = {
+        functions: true,
+        inline: true,
+        reduce_vars: true,
+        toplevel: true,
+        unused: true,
+    }
+    input: {
+        function f() {
+            function g() {
+                return function() {};
+            }
+            var a = g();
+            (function() {
+                this && a && console.log("PASS");
+            })();
+        }
+        f();
+    }
+    expect: {
+        void function() {
+            this && a && console.log("PASS");
+        }();
+        function a() {}
+    }
+    expect_stdout: "PASS"
+}
+
+issue_3371: {
+    options = {
+        functions: true,
+        inline: true,
+        reduce_vars: true,
+        side_effects: true,
+        unused: true,
+    }
+    input: {
+        (function() {
+            var a = function f() {
+                (function() {
+                    console.log(typeof f);
+                })();
+            };
+            while (a());
+        })();
+    }
+    expect: {
+        (function() {
+            function a() {
+                console.log(typeof a);
+            }
+            while (a());
+        })();
+    }
+    expect_stdout: "function"
+}
+
+class_iife: {
+    options = {
+        inline: true,
+        sequences: true,
+        toplevel: true,
+    }
+    input: {
+        var A = function() {
+            function B() {}
+            B.prototype.m = function() {
+                console.log("PASS");
+            };
+            return B;
+        }();
+        new A().m();
+    }
+    expect: {
+        var A = (B.prototype.m = function() {
+            console.log("PASS");
+        }, B);
+        function B() {}
+        new A().m();
+    }
+    expect_stdout: "PASS"
+}
+
+issue_3400: {
+    options = {
+        collapse_vars: true,
+        inline: true,
+        reduce_funcs: true,
+        reduce_vars: true,
+        unused: true,
+    }
+    input: {
+        (function(f) {
+            console.log(f()()[0].p);
+        })(function() {
+            function g() {
+                function h(u) {
+                    var o = {
+                        p: u
+                    };
+                    return console.log(o[g]), o;
+                }
+                function e() {
+                    return [ 42 ].map(function(v) {
+                        return h(v);
+                    });
+                }
+                return e();
+            }
+            return g;
+        });
+    }
+    expect: {
+        void console.log(function g() {
+            function e() {
+                return [42].map(function(v) {
+                    return o = {
+                        p: v
+                    }, console.log(o[g]) , o;
+                    var o;
+                });
+            }
+            return e();
+        }()[0].p);
+    }
+    expect_stdout: [
+        "undefined",
+        "42",
+    ]
+}
+
+issue_3402: {
+    options = {
+        evaluate: true,
+        functions: true,
+        reduce_vars: true,
+        side_effects: true,
+        toplevel: true,
+        typeofs: true,
+        unused: true,
+    }
+    input: {
+        var f = function f() {
+            f = 42;
+            console.log(typeof f);
+        };
+        "function" == typeof f && f();
+        "function" == typeof f && f();
+        console.log(typeof f);
+    }
+    expect: {
+        var f = function f() {
+            f = 42;
+            console.log(typeof f);
+        };
+        f();
+        f();
+        console.log(typeof f);
+    }
+    expect_stdout: [
+        "function",
+        "function",
+        "function",
+    ]
+}
