@@ -59,9 +59,9 @@ do_screw_try_catch: {
     input: {
         good = function(e){
             return function(error){
-                try{
+                try {
                     e()
-                } catch(e) {
+                } catch (e) {
                     error(e)
                 }
             }
@@ -70,9 +70,9 @@ do_screw_try_catch: {
     expect: {
         good = function(n){
             return function(t){
-                try{
+                try {
                     n()
-                } catch(n) {
+                } catch (n) {
                     t(n)
                 }
             }
@@ -93,9 +93,9 @@ dont_screw_try_catch: {
     input: {
         bad = function(e){
             return function(error){
-                try{
+                try {
                     e()
-                } catch(e) {
+                } catch (e) {
                     error(e)
                 }
             }
@@ -104,9 +104,9 @@ dont_screw_try_catch: {
     expect: {
         bad = function(t){
             return function(n){
-                try{
+                try {
                     t()
-                } catch(t) {
+                } catch (t) {
                     n(t)
                 }
             }
@@ -137,7 +137,7 @@ do_screw_try_catch_undefined: {
     }
     expect: {
         function a(o){
-            try{
+            try {
                 throw "Stuff"
             } catch (o) {
                 console.log("caught: "+o)
@@ -172,7 +172,7 @@ dont_screw_try_catch_undefined: {
     }
     expect: {
         function a(n){
-            try{
+            try {
                 throw "Stuff"
             } catch (undefined) {
                 console.log("caught: " + undefined)
@@ -861,3 +861,111 @@ issue_3215_4: {
     }
     expect_stdout: "PASS"
 }
+
+issue_3355_1: {
+    mangle = {
+        ie8: false,
+    }
+    input: {
+        (function f() {
+            var f;
+        })();
+        (function g() {
+        })();
+        console.log(typeof f === typeof g);
+    }
+    expect: {
+        (function o() {
+            var o;
+        })();
+        (function o() {
+        })();
+        console.log(typeof f === typeof g);
+    }
+    expect_stdout: "true"
+}
+
+issue_3355_2: {
+    mangle = {
+        ie8: true,
+    }
+    input: {
+        (function f() {
+            var f;
+        })();
+        (function g() {
+        })();
+        console.log(typeof f === typeof g);
+    }
+    expect: {
+        (function f() {
+            var f;
+        })();
+        (function g() {
+        })();
+        console.log(typeof f === typeof g);
+    }
+    expect_stdout: "true"
+}
+
+issue_3355_3: {
+    mangle = {
+        ie8: false,
+    }
+    input: {
+        !function(a) {
+            "aaaaaaaaaa";
+            a();
+            var b = function c() {
+                var c = 42;
+                console.log("FAIL");
+            };
+        }(function() {
+            console.log("PASS");
+        });
+    }
+    expect: {
+        !function(a) {
+            "aaaaaaaaaa";
+            a();
+            var o = function a() {
+                var a = 42;
+                console.log("FAIL");
+            };
+        }(function() {
+            console.log("PASS");
+        });
+    }
+    expect_stdout: "PASS"
+}
+
+issue_3355_4: {
+    mangle = {
+        ie8: true,
+    }
+    input: {
+        !function(a) {
+            "aaaaaaaaaa";
+            a();
+            var b = function c() {
+                var c = 42;
+                console.log("FAIL");
+            };
+        }(function() {
+            console.log("PASS");
+        });
+    }
+    expect: {
+        !function(a) {
+            "aaaaaaaaaa";
+            a();
+            var o = function n() {
+                var n = 42;
+                console.log("FAIL");
+            };
+        }(function() {
+            console.log("PASS");
+        });
+    }
+    expect_stdout: "PASS"
+}
