@@ -490,6 +490,7 @@ issue_1758: {
 delete_seq_1: {
     options = {
         booleans: true,
+        evaluate: true,
         side_effects: true,
     }
     input: {
@@ -514,6 +515,7 @@ delete_seq_1: {
 delete_seq_2: {
     options = {
         booleans: true,
+        evaluate: true,
         side_effects: true,
     }
     input: {
@@ -538,6 +540,7 @@ delete_seq_2: {
 delete_seq_3: {
     options = {
         booleans: true,
+        evaluate: true,
         keep_infinity: true,
         side_effects: true,
     }
@@ -563,6 +566,7 @@ delete_seq_3: {
 delete_seq_4: {
     options = {
         booleans: true,
+        evaluate: true,
         sequences: true,
         side_effects: true,
     }
@@ -590,6 +594,7 @@ delete_seq_4: {
 delete_seq_5: {
     options = {
         booleans: true,
+        evaluate: true,
         keep_infinity: true,
         sequences: true,
         side_effects: true,
@@ -618,6 +623,7 @@ delete_seq_5: {
 delete_seq_6: {
     options = {
         booleans: true,
+        evaluate: true,
         side_effects: true,
     }
     input: {
@@ -702,7 +708,7 @@ side_effects_cascade_3: {
     }
     expect: {
         function f(a, b) {
-            !(b += a) && ((b = a) || (b -= a, b ^= a)),
+            (b += a) || (b = a) || (b -= a, b ^= a),
             a--;
         }
     }
@@ -964,3 +970,39 @@ missing_link: {
         console.log(a);
     }
 }
+
+angularjs_chain: {
+    options = {
+        conditionals: true,
+        sequences: true,
+        side_effects: true,
+    }
+    input: {
+        function nonComputedMember(left, right, context, create) {
+            var lhs = left();
+            if (create && create !== 1) {
+                if (lhs && lhs[right] == null) {
+                    lhs[right] = {};
+                }
+            }
+            var value = lhs != null ? lhs[right] : undefined;
+            if (context) {
+                return { context: lhs, name: right, value: value };
+            } else {
+                return value;
+            }
+        }
+    }
+    expect: {
+        function nonComputedMember(left, right, context, create) {
+            var lhs = left();
+            create && 1 !== create && lhs && null == lhs[right] && (lhs[right] = {});
+            var value = null != lhs ? lhs[right] : void 0;
+            return context ? {
+                context: lhs,
+                name: right,
+                value: value
+            } : value;
+        }
+    }
+}
